#include "inclusive.h"
#include "PWMIO.h"

#define OPTIONS "i:o:w:p:vh?"

// forward declaration function
void instructions();
void version();
void cleanup();

string *inputFile = NULL,
       *outputFile = NULL;
double *pseudo = NULL;

int
main(int argc, char *argv[])
{
  char c;

  int wLength = 8;
  pseudo = new double[4];
  for(int i = 0; i < 4; i++) pseudo[i] = 0.001;
  
  if (optopt == 0)
  { instructions(); exit(-1);}

  while ((c = getopt(argc, argv, OPTIONS)) != -1)
  {
    string *pseudoinput = NULL;
    string::size_type pos;
    switch (c)
    {
    case 'i':
      // set input file name
      inputFile = new string(optarg);
      break;
    case 'o':
      // define new output stream 
      outputFile = new string(optarg);
      break;
    case 'w':
      wLength = atoi(optarg);
      if ( wLength < 6 || wLength > 50)
      {
        cerr << "--ERROR--BuildPWM : -w should be between 6 and 50." << endl;
        cleanup(); exit(-1);
      }
      break;
    case 'p':
      pseudoinput = new string(optarg); pseudoinput->append("_");
      double v;
      for(int i = 0; i < 4; i++)
      {
        pos = pseudoinput->find_first_of("_");
        if (pos != string::npos)
        { istringstream istr(pseudoinput->substr(0,pos));
          istr >> v;
          if (v < 0 || v > 10)
          { cerr << "--ERROR--BuildPWM : -p has invalid numeric input" << endl;
            delete pseudoinput; cleanup(); exit(-1);
          }
          pseudo[i] = v;
          pseudoinput->erase(0,pos+1);
        }
        else
        { cerr << "--ERROR--BuildPWM : -p invalid format" 
               << " <value_value_value_value>" << endl;
          delete pseudoinput; cleanup(); exit(-1);
        }
      }
      break;
    case 'v':
      version();exit(1);
    case '?':
    case 'h':
      instructions();
      exit(1);
    default:
      cerr << "BuildPWM: Error in getopt() function" << endl;
      exit(-1);
    }
  }


  if (inputFile == NULL || outputFile == NULL)
  { instructions(); cleanup(); exit(-1);}

  // open inputfile for reading
  ifstream _ifs;
  string _pLine; string * pSeqStr = NULL;
  _ifs.open(inputFile->c_str(), ios::in);
  int count = 1;

  // open outfile for writing
  PWMIO* pPwmIO = new PWMIO(outputFile, WRITE); 
  PWM * pMotif = NULL;
  double** pCounts = new double*[wLength];
  for(int i = 0; i < wLength; i++) 
  { pCounts[i] = new double[4];
    for(int n = 0; n < 4; n++) pCounts[i][n] = 0;
  }

  if (!_ifs)
  {
    // file is not open for reading
    cerr << "--Error--BuildPWM unable to open file "
      << *inputFile << " for reading." << endl;
   cleanup(); exit(-1);
  }
  // read lines untill first line starting with a '#' is found 
  do
  { getline(_ifs, _pLine, '\n');}
  while (_pLine[0] != '#' && !_ifs.eof());

  if (_ifs.eof()) 
  {
    cerr << "--Error--BuildPWM file " << *inputFile 
         << " does not have # lines." << endl;
    cleanup(); exit(-1);
  }

  // read the instances on each line
  while (!_ifs.eof())
  {
    // read next line
    getline(_ifs, _pLine, '\n');
    // move to next line if line is empty
    if (_pLine.empty() && !_ifs.eof()) continue;
    // start next motif at #
    if (_pLine[0] == '#' || _ifs.eof())
    {
      // create earlier PWM
      pMotif = new PWM(wLength, pCounts, pseudo);
      // set ID
      ostringstream id;
      id << "box_" << count << "_ps" << pseudo[0] << "_" 
         <<  *(pMotif->GetConsensus());
      pMotif->SetID(id.str());
      // and print earlier PWM
      pPwmIO->WriteMatrix(pMotif); 
      delete pMotif; pMotif = NULL;
      // reset counts for next PWM
      count++; 
      for(int i = 0; i < wLength; i++) 
      { for(int n = 0; n < 4; n++) pCounts[i][n] = 0;}
    }
    else
    { // read instance
      pSeqStr = new string(_pLine);
      int extra = (int)pSeqStr->length()- wLength;
      if ( extra > 0)
      { cerr << "--Warning--BuildPWM : instance in file longer than " << wLength 
               << " / skipping " <<  extra << " longer counts." << endl;
      }
      if ( extra < 0)
      {cerr << "--Warning--BuildPWM : instance in file shorter than " << wLength 
               << " / adding " << -extra << " neutral counts." << endl;
      }
      // transform and add to counts (A=0,C=1,G=2,T=3)
      for (int i = 0; i < wLength; i++) 
      {
        if (i >= (int)pSeqStr->length())
        { for(int n = 0; n < 4; n++) pCounts[i][n] += 0.25; 
          //cerr << "Warning::adding neutral counts at position " << i << endl;
        }
        else
        {
          switch (pSeqStr->at(i))
          {
            case 'a': case 'A':
              pCounts[i][0]++;
              break;
            case 'c': case 'C':
              pCounts[i][1]++;
              break;
            case 'g': case 'G':
              pCounts[i][2]++;
              break;
            case 't': case 'T':
              pCounts[i][3]++;
              break;
            default:
              for(int n = 0; n < 4; n++) pCounts[i][n] += 0.25;
              cerr << "Warning::unvalid nt::adding neutral counts at position " << i << endl;
          }
        }
      }
      // cleanup
      delete pSeqStr; pSeqStr = NULL;
    }
  }
  // close the files
  _ifs.close(); 
  delete pPwmIO; pPwmIO = NULL;
  // ending
  cleanup();
  cerr << "#Exit program succesfully." << endl; 
  exit(0);
}



/*********************
 *  LOCAL FUNCTIONS  *
 *********************/
void
instructions()
{
  cout << endl;
  cout << "Usage:" << " BuildPWM (internal use)" << endl;
  cout << endl;
  cout << " Required Arguments" << endl;
  cout << "  -i <inputFile>      Sites_list.txt format format" << endl;
  cout << "  -o <outputFile>     " << endl;
  cout << endl;
  cout << " Optional Arguments" << endl;
  cout << "  -w <value>          Motiflength, default<8>" << endl;
  cout << "  -p <values_>        Pseudocount, default<0.001_0.001_0.001_0.001>" << endl;
  cout << endl;
  cout << "Version " << VERSION << endl;
  cout << "Questions and Remarks: <mclaeys@qatar.net.qa>" << endl << endl;
} 

void
version()
{
  cout << endl;
  cout << "INCLUSive - BuildPWM (internal use) " << endl;
  cout << "  version " << VERSION << endl;
  cout << endl;
  cout << "  Revision history : " << endl;
  cout << "  - 08/03/09 : increased range of user parameter -w, " 
       << " revised some error messages." << endl;
  cout << "- (6.1.0) - 2010/11/10 : zero matrix contributes zero in CS (else nan) "<< endl;
  cout << "- (4.1.2) - 2014/02/15 : transfer within INCLUSive (no functional change)." << endl;
  cout << "  - end." << endl;
  cout << endl;
}

void 
cleanup()
{
  // cleanup variables
  if (inputFile != NULL)
    delete inputFile;
  inputFile = NULL;
  if (outputFile != NULL)
    delete outputFile;
  outputFile = NULL;
  if (pseudo != NULL)
    delete[] pseudo;
  pseudo = NULL;
}
