#include "NOrthoSamplerRun.h"
#include "RandomNumber.h"
#include "PWMIO.h"
#include <getopt.h>
#include <sstream> 

#define OPTIONS "b:c:f:hi:j:k:m:M:n:N:o:p:q:Q:r:s:w:x:X:Y:Z:?v"

// function prototypes
void instructions();
void version();
void cleanup();

// initialize empty file pointers
string *fastaFile = NULL;
string *bgFile = NULL;
string *treeFile = NULL;
string *priorFile = NULL;
string *pspFile = NULL;
string *outputFile = NULL;
string *matrixFile = NULL;
string *outputFile_spec = NULL;
string *matrixFile_spec = NULL;

// mainprogram
int main(int argc, char *argv[])
{ 
  // initialize random generator
  RandomNumber::InitRandomNumber();
  srandom((long)time(NULL));
  cerr << "NOrthoMotifSampler Statistical Seed = " << random() << endl;
  // local variables
  char c;
  // defaults if optional argument is not given
  strand_modes strand = both; 
  string _pDefault = "0.9_0.25";
  bool _bPSP_s = 0; // default only PSP for updating step
  int wLength(8), nbrRuns(100), nbrMotifs(1), MotifOverlap(1), maxNbrInst(2); 
  double dMinSeqInst(0.5);
  double dMaxMasked(0.2);
  double dSlowMotifRate(0.5);
  int iPriorScale(1);
  ostringstream cerrstr; // to output all error messages (instead of cerr)
  bool wronginput = false;
  bool bOut = false;
  int NbrIter[3] = {20,70,10};
  int PrctStart[2] = {100,50}; int PrctEnd[2] = {100,10}; // not used !!
  // 2014/01/22 : also print species-specific motifs as separate output
  bool bSpecific = 0; // output also species-specific motif results

 
  // process the mainprogram input arguments
  if (optopt == 0)
  {
    instructions();
    exit(-1); // program ends without success
  }
  while ((c = getopt(argc, argv, OPTIONS)) != -1)
  {
    string *iterations = NULL;
    switch (c)
    {
    case 'f': // set fasta file name
      fastaFile = new string(optarg);
      break;
    case 'b': // set background file name
      bgFile = new string(optarg);
      break;
    case 'c': // set tree file name
      treeFile = new string(optarg);
      break;
    case 'o': // define output stream to write instance results 
      bOut = true;
      outputFile = new string(optarg);
      break;
    case 'm': // define matrix file to store motif models
      matrixFile = new string(optarg);
      break;
    case 'i': // set fraction of sequences with at least one instance
      dMinSeqInst = (double)atof(optarg);
      if ( dMinSeqInst <= 0 || dMinSeqInst > 1)
      {
        cerr << "--ERROR--NOrthoMotifSampler: -i input should be "
             << "between 0 and 1." << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -i input should be "
             << "between 0 and 1." << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'j': // slower motif evolution rate
      dSlowMotifRate = (double)atof(optarg);
      if ( dSlowMotifRate <= 0 || dSlowMotifRate > 1)
      {
        cerr << "--ERROR--NOrthoMotifSampler: -m input should be "
             << "between 0 and 1." << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -m input should be "
             << "between 0 and 1." << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'k': // iPriorScale : balance of prior evolution impact
      iPriorScale = (int)atoi(optarg);
      if ( iPriorScale <= 0 )
      {
        cerr << "--ERROR--NOrthoMotifSampler: -k input should be "
             << "higher than 0." << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -m input should be "
             << "higher than 0." << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'M': // set maximum nbr of instances
      maxNbrInst = atoi(optarg);
      if ( maxNbrInst < 1 )
      {
        cerr << "--ERROR--NOrthoMotifSampler: -M maximal number of instances per sequence should be "
             << "> 0." << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: M maximal number of instances per sequence should be "
             << "> 0." << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'n': // set # motifs
      nbrMotifs = atoi(optarg);
      if (nbrMotifs < 1)
      {
        cerr << "--ERROR--NOrthoMotifSampler: -n number of different motifs should be > 0." 
             << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -n number of different motifs should be > 0." 
             << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'N': // set maximum masked fraction of a sequence
      dMaxMasked = (double)atof(optarg);
      if (dMaxMasked < 0 || dMaxMasked >= 1)
      {
        cerr << "--ERROR--NOrthoMotifSampler: -X MaxMasked fraction should "
             << "be between 0 and 1." << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -X MaxMasked fraction should "
             << "be between 0 and 1." << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'p': // read prior on number of instances per sequence
      priorFile = new string(optarg);
      break; 
    case 'q':
      pspFile = new string(optarg);
      break;
    case 'Q': // for test purposes
      _bPSP_s = (bool)atoi(optarg);
      break;
    case 'r': // set # runs
      nbrRuns = atoi(optarg);
      if (nbrRuns < 1)
      {
        cerr << "--ERROR--NOrthoMotifSampler: -r number of runs should be > 0." 
             << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -r number of runs should be > 0." 
             << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 's': // set strand mode
      if (atoi(optarg) == 1)
        strand = both; 
      else if (atoi(optarg) == 0)
        strand = plus_strand; 
      else 
      {
        cerr << "--ERROR--NOrthoMotifSampler: -s strand should be 0 or 1." 
             << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -s strand should be 0 or 1." 
             << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'w': // set motif length
      wLength = atoi(optarg);
      if (wLength <= 4 || wLength >= 30)
      {
        cerr << "--ERROR--NOrthoMotifSampler: -w motif width should be > 4" <<
               " and < 30." << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -w motif width should be > 4" <<
               " and < 30." << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'x': // set allowed overlap
      MotifOverlap = atoi(optarg);
      if (MotifOverlap < 1)
      {
        cerr << "--ERROR--NOrthoMotifSampler: -x motifoverlap should be > 0." 
             << endl;
        cerrstr << "--ERROR--NOrthoMotifSampler: -x motifoverlap should be > 0." 
             << endl;
        wronginput = true;
        //cleanup(); exit(-1);
      }
      break;
    case 'X': // set number of iterations for each step // TEST
      iterations = new string(optarg); 
      double p;
      for(int i = 0; i < 3; i++)
      {
        if( iterations->find_first_of("X") == 0) 
        { // leave default for this step
          iterations->erase(0,1); // erase X
          if (i == 2 && iterations->empty()) break;
          if( iterations->find_first_of("_") == 0) 
          { iterations->erase(0,1); continue; }
        }
        if (iterations->empty())
        { cerr << "--ERROR--NOrthoMotifSampler: -X iterations format not "
          << "correct.\n"; delete iterations; cleanup(); exit(-1);
        }
        if (iterations->find_first_of("[") != 0)
        {// read the number for this step
          p = PHYLINCLUSIVE::ReadValue(*iterations); NbrIter[i] = (int)p;
          if ( p < 0)         
          { cerr << "--ERROR--NOrthoMotifSampler: -X iterations value should "
               << "be > 0.\n"; delete iterations; cleanup(); exit(-1);
          }
        }
        if (iterations->find_first_of("[") == 0)
        { // for core/conv also check for [.,.] input
          iterations->erase(0,1); // erase [
          for(int j = 0; j < 2; j++)
          {
            if( iterations->find_first_of("X") == 0)
            { iterations->erase(0,2);} // erase X, or X]
            else
            { 
              p = PHYLINCLUSIVE::ReadValue(*iterations); 
              if ( p <= 0 || p > 100 )         
              { cerr << "--ERROR--NOrthoMotifSampler: -X [pcrt,prct] value "
                     "should be between 0 and 100.\n"; 
                delete iterations; cleanup(); exit(-1);
               }
              if (j == 0) PrctStart[i-1] = (int)p; 
              else PrctEnd[i-1] = (int)p;
              iterations->erase(0,1); // erase , or ]
            }
          }
          if( iterations->find_first_of("_") == 0) iterations->erase(0,1); // _
        }
      }
      delete iterations; iterations = NULL;
      break;
    case 'Z': // 2014/01/22 - output species-specific motifs
      bSpecific = (bool)atoi(optarg);
      break;
    case '?':
    case 'h':
      instructions(); exit(-1);
    case 'v':
      version(); exit(-1);
    default:
      cerr << "-- NOrthoMotifSampler: Error in getopt() function" << endl;
      cerrstr << "--NOrthoMotifSampler: Error in input getopt() function" << endl; 
      wronginput = true;
      //cleanup(); exit(-1);
    }
  }

  // check required input is avaliable if not report/exit
  GFFWriter *pGffIO = NULL;
  if (fastaFile == NULL || bgFile == NULL || treeFile == NULL || !bOut)
  {
    cerr << "--ERROR: required input/outputfiles (-f, -b, -c, -o|-m) are not provided." << endl;
    cerrstr << "--ERROR: required input/outputfiles (-f, -b, -c, -o|-m) are not provided." << endl;
    // report to file if file is available
    if (bOut) 
    { cerrstr << "#Check our MotifSuite webpage for correct input format. Or contact us, we will be happy to help." << endl;
      pGffIO = new GFFWriter(outputFile); 
      pGffIO->AddComment(cerrstr.str());
      cerrstr.flush(); // flush  
      if (pGffIO != NULL) delete pGffIO; pGffIO = NULL; 
    }
    // exit
    instructions(); cleanup(); exit(-1); 
  }

  if (wronginput)
  { 
    // report to file if file is available
    if (bOut) 
    { cerrstr << "#Check our MotifSuite webpage for correct input format. Or contact us, we will be happy to help." << endl;
      pGffIO = new GFFWriter(outputFile); 
      pGffIO->AddComment(cerrstr.str());
      cerrstr.flush(); // flush  
      if (pGffIO != NULL) delete pGffIO; pGffIO = NULL;
    }
    // exit
    cleanup(); exit(-1);  
  }

  // the program has the requested input and can proceed
  
  /* INITIALIZE OBJECTS TO WRITE RESULTS -------------------------------------
  ----------------------------------------------------------------------------*/
  // initialize file to write motif models
  PWMIO *pPwmIO = NULL; 
  // 2014/01/22 
  PWMIO *pPwmIO_spec = NULL; GFFWriter * pGffIO_spec = NULL; 
  if (matrixFile != NULL) // this means it is intended to have PWM reporting
  {
    pPwmIO = new PWMIO(matrixFile, WRITE);
    if (pPwmIO->GetError() != NULL)
    {
      cerr << *(pPwmIO->GetError());
      if (bOut)
      {
        pGffIO = new GFFWriter(outputFile); 
        pGffIO->AddComment(pPwmIO->GetError()); 
        delete pGffIO; pGffIO = NULL;  
      }
      delete pPwmIO; pPwmIO = NULL;
      cleanup(); exit(-1);  
    }
    // 2014/01/22
    if (bSpecific)
    { // create a new mtrx-file based on user-matrixfile
      matrixFile_spec = new string(*matrixFile);
      matrixFile_spec->insert(matrixFile_spec->rfind("."),"_spec");
      pPwmIO_spec = new PWMIO(matrixFile_spec, WRITE);
    }
  }
  // define output stream to write GFF results // always ok
  if (bOut)
  {
    pGffIO = new GFFWriter(outputFile);
    // 2014/01/22
    if (bSpecific)
    { // create a new txt-file based on user-txtfile
      outputFile_spec = new string(*outputFile);
      outputFile_spec->insert(outputFile_spec->rfind("."),"_spec");
      pGffIO_spec = new GFFWriter(outputFile_spec);
    }
  }
  else
  {
    pGffIO = new GFFWriter();
  }

  NOrthoSamplerRun *pMainNOrthoRunner = NULL;
  /* CREATE NORTHOSAMPLER-OBJECT WHERE SEQUENCES ARE LOADED--------------------
  ---------------------------------------------------------------------------*/
  cerr << "Create NOrthoMotifSampler run." << endl;
  try 
  {
    // create an object and pass on the required inputfiles
    pMainNOrthoRunner = new NOrthoSamplerRun();
    cerr << "#Loading sequences " << endl;
    pMainNOrthoRunner->LoadSequences(fastaFile, strand);
    // load the backgroundmodels
    cerr << "#Loading backgroundmodels " << endl;
    pMainNOrthoRunner->LoadBackgroundModels(bgFile);

    // load the correlations
    cerr << "#Loading correlations " << endl;
    string reporttree = pMainNOrthoRunner->LoadCorrelations(treeFile);

    cerr << "#Loading parameters " << endl;
    // check the overlap -x (no need to store in pMainRunner)
    if (MotifOverlap > wLength) MotifOverlap = wLength;
    // processing of -p (#inst/seq) information
    pMainNOrthoRunner->SetNbrInstPrior(maxNbrInst, priorFile, _pDefault);
    // some preparations for running motifdetection
    pMainNOrthoRunner->InitializeLocalVariables1(wLength);
    // some preparations for running motifdetection
    pMainNOrthoRunner->InitializeLocalVariables2(dSlowMotifRate, iPriorScale);
    // processing of -q (PSP) information.
    if (pspFile != NULL) { pMainNOrthoRunner->LoadPSP(pspFile, _bPSP_s);}
  
  /* RUN THE MOTIF DETECTION ALGORITHM----------------------------------------
  ---------------------------------------------------------------------------*/
    string * intro = new string("#see end of file for a summary of results.\n#The multiple motifs in this file are the result of stochastic motif detection repeats on a given sequences dataset.\n#Please proceed with extraction of the most likely true motif(s) embedded in this file by using MotifRanking (quick assessment) or FuzzyClustering (more refined assessment). Both post processing tools are available in MotifSuite on our webserver on which you can also consult our guidelines.\n");
    intro->append(reporttree);
    pGffIO->AddComment(intro);
    delete intro; intro = NULL;

   // string *pSource = new string("NOrthoMotifSampler");
	  
    // run the motif detection algorithm
    time_t t1; time(&t1);
    cerr << "#Searching for " << nbrRuns << " x " << nbrMotifs << " motifs..." << endl;

    int errorSampling = 0;
    int errorMasking = 0;
    int retrievedMotifs = 0;
int minSeq(INT_MAX), maxSeq(0), minInst(INT_MAX), maxInst(0),minLL(INT_MAX), maxLL(0);
double minCS(2.1), maxCS(0), minIC(2.1), maxIC(0), score;

    for (int r = 1; r <= nbrRuns; r++)
    { //cerr << "\n----------------------------------------- run: " << r << endl;

      // reset mask (all 1)
      if (nbrMotifs > 1 && r != 1)
      { //cerr << "Resetting mask." << endl; 
        pMainNOrthoRunner->ResetMasks();
      } //cerr << endl;

      int m = 1;
      try 
      {
        for (; m <= nbrMotifs; m++)
        { 
          try
          {
            cerr << "Searching for motif number: " << r << "." << m << endl;
            // initial start with motif instances from uniform distribution
            //cerr << "Initial start" << endl;
            pMainNOrthoRunner->InitInstanceMap();
            // initialize motif instances from random distribution
            //cerr << "Initialization steps " << endl;
            pMainNOrthoRunner->DetectionStep("init", NbrIter[0]); 
            // core motif sampler iterations
            //cerr << "Core sampling steps "<< endl;
            pMainNOrthoRunner->DetectionStep("core", NbrIter[1]); 
            // convergence iterations
            //cerr << "Convergence steps " << endl;
            pMainNOrthoRunner->DetectionStep("conv", NbrIter[2]); 
/*
			  GFFWriter * pScreen = new GFFWriter;
			  pMainNOrthoRunner->PrintMotifInstances(pScreen);
			  delete pScreen; pScreen = NULL;
			  
*/
            // evaluate results of this run
            // cerr << "Evaluating results for motif "<< r << "."<< m << endl;
            int * n1n2 = pMainNOrthoRunner->EvaluateMotif(dMinSeqInst, "all");
            // print motif id/scores, instances and matrices 
            // cerr << "Printing results for motif " << r << "." << m << endl; 
            PWM * pMotif = pMainNOrthoRunner->BuildMotifFromSpeciesInstanceMap(0, "all");
            retrievedMotifs++;
			  
            // 2014/01/22 : printing to output - move this to NOSamplerRun
            ostringstream id;
            id << "box_"<< r <<"_"<< m<< "_" << *(pMotif->GetConsensus());
            pMotif->SetID(id.str());
            ostringstream info;
            info << "nbrSeq: " << n1n2[0] << "\tnbrInst: " << n1n2[1];
            pMotif->SetInfo(info.str());
            // print all
            // 2014/01/21
            // print also species-specific-PWMs as output to the user
            pMainNOrthoRunner->PrintToOutput(pMotif, pGffIO, pPwmIO);
            if (bSpecific)
            { pMainNOrthoRunner->PrintToOutput(pMotif, pGffIO_spec, pPwmIO_spec);
              ostringstream boxid;
              boxid << "box_"<< r <<"_"<< m;
              pMainNOrthoRunner->PrintToSpecOutput(boxid.str(), pGffIO_spec, pPwmIO_spec);
              boxid.flush();
            }

// track min/max scores
score = (int)n1n2[0];
if (score <= minSeq) { minSeq = score;}
if (score >= maxSeq) { maxSeq = score;}
score = (int)n1n2[1];
if (score <= minInst) { minInst = score;}
if (score >= maxInst) { maxInst = score;}
score = pMotif->GetCS();
if (score <= minCS) { minCS = score;}
if (score >= maxCS) { maxCS = score;}
score = pMotif->GetIC();
if (score <= minIC) { minIC = score;}
if (score >= maxIC) { maxIC = score;}
score = (int)pMotif->GetLL();
if (score <= minLL) { minLL = score;}
if (score >= maxLL) { maxLL = score;}

            // cleanup
            delete[] n1n2; n1n2 = NULL;
            id.flush(); info.flush();// flush
            delete pMotif; pMotif = NULL;

            // apply mask based on retrieved instance map
            if (nbrMotifs > 1 && m != nbrMotifs)
            { cerr << "Applying mask on retrieved instances"  << endl << endl;
              pMainNOrthoRunner->UpdateMasks(dMaxMasked, MotifOverlap);
            }
          } // end of try-block for one motif
          catch(AbortSampling){ errorSampling++;  
                            cerr <<"Ignore motif " << r << "." << m << endl;
			  ostringstream out_1; 
			  out_1 << "#AbortSampling - Ignore motif " << r << "." << m << endl;
			  pGffIO->AddComment(out_1.str());
              out_1.flush(); // flush
          }
        }
      } // end of try-block for one run
      catch(AbortMasking){ errorMasking++; 
                       cerr <<"Abort run " << r << "at motif " << m << endl;
			ostringstream out_2; 
			out_2 << "#AbortMasking - Abort run " << r << "at motif " << m << endl;
			pGffIO->AddComment(out_2.str());
            out_2.flush(); // flush
	  }
    }
    // cerr << endl;
			ostringstream out_3;
	//// report resume information
out_3 << "# NOrthoMotifSampler results summary :\n";
out_3 << "# number of reported motifs :" << retrievedMotifs <<"\n";
out_3 << "# convergence rate :" << int(100*retrievedMotifs/nbrRuns/nbrMotifs) <<"%\n";
out_3 << "# maximal/minimal number of sequences with a motif : " 
		<< maxSeq << "/" << minSeq << "\n";
out_3 << "# maximal/minimal number of instances in a motif : " 
		<< maxInst << "/" << minInst << "\n";
out_3 << "# maximal/minimal motif Consensus score : " 
		<< setprecision(3) << maxCS << "/" << setprecision(3) << minCS << "\n";
out_3 << "# maximal/minimal motif Information Content score : " 
		<< setprecision(3)<< maxIC << "/" << setprecision(3) << minIC << "\n";
out_3 << "# maximal/minimal motif LogLikelihood score : " 
		<< maxLL << "/" << minLL << ".\n";
	
if (errorSampling>= 10 && errorSampling >= (int) 0.2 * (nbrRuns-errorMasking)*nbrMotifs)
{ out_3 << "# " << errorSampling << " predictions were rejected because of too few "
<< "retrieved instances. You may want to repeat the program with a higher "
<< "probability for number of instances per sequence (-p)." << endl;
}
if (errorMasking>= 10 && errorMasking >= (int)0.2 * nbrRuns)
{ out_3 << "# " << errorMasking << " runs were aborted for searching a next motif most probably because of too much "
<< "sequence masking by earlier retrieved motifs in this run. "
<< "Verify if -N is not set unreasonably high. Alternatively, you may want to "
<< "repeat the program with a lower -n (number of motifs) yet higher -r "
<< "(number of runs) and explore more of the top ranked motifs (use MotifRanking).\n";
}
//out_3 << endl;
    // ending
    time_t t2; time(&t2);
    // add time to outputFile
out_3 << "# NOrthoMotifSampler runtime information : ";
    //out_3 << "# Date :"  << ctime(&t1);
    double T = difftime(t2, t1);
    out_3 << (int)T << " sec (=~ " << (int)T/3600 << " hrs, " 
    << int((T - ((int)T/3600)*3600)/60) << " min)" << endl; 
	pGffIO->AddComment(out_3.str());
	cerr << out_3.str(); // also print on screen
    out_3.flush(); // flush
	// ending
    cerr << "#End of motif detection algorithm" << endl;  
  } // end of overall try-block
  catch (ErrorInputfiles)
  { cerr << "--Error--catch ErrorInputfiles: Ending program without results." << endl;
    cerrstr << "--Error--catch ErrorInputfiles: Ending program without results." << endl;
    pGffIO->AddComment(cerrstr.str());
    cerrstr.flush(); // flush  
  }
  catch (InputDimension)
  { cerr << "--Error--catch ErrorDimension: Ending program without results." << endl;
    cerrstr << "--Error--catch ErrorDimension: Ending program without results." << endl;
    pGffIO->AddComment(cerrstr.str());
    cerrstr.flush(); // flush  
  }

  /* cleanup variables and close outputfiles--------------------------------*/
  if (pMainNOrthoRunner!= NULL) delete pMainNOrthoRunner; pMainNOrthoRunner = NULL;
  if (pGffIO != NULL) delete pGffIO; pGffIO = NULL;
  if (pPwmIO != NULL) delete pPwmIO; pPwmIO = NULL; 
  if (pGffIO_spec != NULL) delete pGffIO_spec; pGffIO_spec = NULL;
  if (pPwmIO_spec != NULL) delete pPwmIO_spec; pPwmIO_spec = NULL;
  cleanup();
  cerr << "#Exit program succesfully." << endl; 
  exit(0);
}  // end of main program


/***************** definition function protoypes *********************/
void
instructions()
{
  cout << endl;
  cout << "Usage: NOrthoMotifSampler " << endl;
  cout << endl;
  cout << " Required Arguments" << endl;
  cout << " -f <fastaFile>       Fasta format, orthologous sequences grouped " 
                                 << "per gene." << endl;
  cout << " -b <Bgfile>          List of species with BackgroundModel-filename." 
                                 << endl;
  cout << " -c <Corrfile>        Rooted phylogenetic tree with branchlengts and "
                                 "optional parameters." << endl;
  cout << endl;
  cout << " Optional Arguments" << endl;
  cout << " -i <value>           Sets minimal fraction of sequences with at least one motif instance (default 0.5)." << endl;
  cout << " -j <value>           Sets the fractional slower motif evolution rate compared to background (default 0.5)." << endl;
  cout << " -k <value>           Sets the proportional weight of prior evolution counts compared to data-inferred evolution (default 1)." << endl;
  cout << " -M <value>           Sets maximal number of instances per sequence (default 2).\n";
  cout << " -n <value>           Sets number of motifs to search for (default 1)." << endl;
  cout << " -N <value>           Sets maximal masked fraction of a sequence at start of a motif search (default 0.2)." << endl;
  cout << " -p <file>            Specifies the prior distribution on number of instances per sequence per species (default all 0.9_0.25)."<< endl;
  cout << " -q <pspFile>         File containing Position Specific Prior information."<< endl;
  cout << " -Q <0|1>             To apply psp-correction in sampling step (1=yes) (default 0).\n";
  cout << " -r <runs>            Set number of times the algorithm should be repeated (default 100)." << endl;
  cout << " -s <0|1>             Select strand (default both)." << endl;
  cout << "                      0 is only input sequences, 1 includes reverse complement."<< endl;
  cout << " -w <value>           Sets the motif width (default 8)." << endl;
  cout << " -x <value>           Sets allowed overlap between different motifs (default 1)." << endl;
  cout << "#-X                   For test purposes - default -X<20_70[100,50]_10[100,10]>\n";
  cout << " -Z <0|1>             Output also species-specific motif results (default 0).\n ";
  cout << endl; 
  cout << " Output Arguments" << endl;
  cout << " -o <outputFile>     Output file to write detected motifs in instances format." << endl;
  cout << " -m <matrixFile>     Output file to write detected motifs in pwm format." << endl;
  cout << endl;
  cout << "Version " << VERSION << endl;
  cout << "Questions and Remarks: <mclaeys@qatar.net.qa>" << endl << endl;
} 

/***************** definition function protoypes *********************/
void
version()
{
  cout << endl;
  cout << "INCLUSive -- NOrthoMotifSampler (stand alone C++ version)" << endl;
  cout << "Version " << VERSION << endl;
  cout << "Revision history : " << endl; 
  cout << "- 20/11/06 : pseudocount for nodes/root, based on background-SNF. " 
       << endl;
  cout << "- 21/11/06 : normalize LL-score " << endl; 
  cout << "- 08/03/07 : building local PWM see TN07, " << endl 
       << "             node-sampling with apriori-algorithm."<< endl;
  cout << "- 11/03/07 : optional output of internal information."<< endl;
  cout << "- 21/06/07 : see TN05-rev3, main changes :  " << endl 
       << "             optional coregulation in nodes/species," << endl  
       << "             limit runtime of apriori-algorithm, " << endl 
       << "             merge NOrthoMotifRanking into NOrthoMotifSampler, " << endl
       << "             TO-DO-output centered instances solution." << endl; 
  cout << "- 23/08/07 : replace apriori by column node sampling, " << endl
       << "             encrease number of iterations (50,150,30), " << endl
       << "             initialization node instances uniformly random, " << endl
       << "             bug fixing initial sampling iterations, " << endl
       << "             explicit k-parameter for multiple instances distribution," << endl
       << "             evaluation shifting on species-instances instead of total-instances. " << endl; 
  cout << "- 02/01/08 : 1 bug fixed : UpdateMasksFromInstanceMape()." << endl;
  cout << "- 19/11/2012 : Revamp." << endl;
  cout << "- (4.1.0) - 2013/03/12 : checkup/revision PSP code (**)." << endl;
  cout << "- (4.1.0) - 2013/03/24 : back to MULTIPLY ipv ADD (PWM::ApplyEvolPWMs)." << endl;
  cout << "- (4.1.0) - 2013/05/15 : debug segm fault for too short sequences." << endl;
  cout << "- (4.1.0) - 2013/06/10 : revision load phylogenetic tree (preprocessing)." << endl;
  cout << "- (4.1.0) - 2013/06/18 : debug code for gene-specific sequence absenses (*)." << endl;
  cout << "- (4.1.1) - 2014/01/19 : revamp (*)::NOSeqComp = NULL; (**)::as in MS-PSP." << endl;
  cout << "- (4.1.1) - 2014/01/23 : output also species-specific PWMs (-Z)." << endl;
  cout << "- (4.1.1) - 2014/01/30 : document -Q to be boolean." << endl;
  cout << "- (4.1.2) - 2014/02/15 : no functional changes for NOrthoMotifSampler." << endl;
  cout << endl; 
} 

/***************** definition function protoypes *********************/
void
cleanup()
{
  if (fastaFile != NULL)
    delete fastaFile;
  fastaFile = NULL;
  
  if (bgFile != NULL)
    delete bgFile;
  bgFile = NULL;
  
  if (treeFile != NULL)
    delete treeFile;
  treeFile = NULL;
  
  if (priorFile != NULL)
    delete priorFile;
  priorFile = NULL;

  if (pspFile != NULL)
    delete pspFile;
  pspFile = NULL;  

  if (outputFile != NULL)
    delete outputFile;
  outputFile = NULL;
  
  if (matrixFile != NULL)
    delete matrixFile;
  matrixFile = NULL;
	
  if (outputFile_spec != NULL)
    delete outputFile_spec;
  outputFile_spec = NULL;
  
  if (matrixFile_spec != NULL)
    delete matrixFile_spec;
  matrixFile_spec = NULL;
}
