#include "InstanceMap.h"
#include "inclusive.h"


/******************************************************************************
  Method:       new
  Class:        InstanceMap 
  Arguments:    none
  Description:  constructor
	              creates an empty list of Instance objects
  Date:         MC_2014/02/15
******************************************************************************/
InstanceMap::InstanceMap()
{
  // create empty list
  _pInstanceList = new list < Instance * >;
  _numberOfInstances = 0; 
  _pW = NULL; //
  _pInstanceCounts = NULL; // 
}
/******************************************************************************
  Description:  constructor (PhyloMotifSamplerRun)
  Author_Date:  MC_2014/02/15
******************************************************************************/
InstanceMap::InstanceMap(int *pW)
{
  // create empty list
  _pInstanceList = new list < Instance * >;
  _numberOfInstances = 0;
  _pW = pW; //
  _pInstanceCounts = new StringVector**[*_pW]; //
  for (int w = 0; w < *_pW; w++)
  {
    _pInstanceCounts[w] = new StringVector*[4]; 
    for (int n = 0; n < 4; n++)
      _pInstanceCounts[w][n] = new StringVector;
  }
}
/******************************************************************************
  Method:       delete
  Class:        InstanceMap
  Arguments:    none
  Description:  destructor
  Date:         MC_2014/02/15
******************************************************************************/
InstanceMap::~InstanceMap()
{
  // first delete all individual instances in the list
  list < Instance * >::iterator lIter = _pInstanceList->begin();
  while (lIter != _pInstanceList->end())
  {
    if ( (*lIter) != NULL )
      delete(*lIter);
    lIter++;
  }
  // delete the list itself
  delete _pInstanceList;
  _pInstanceList = NULL;
  _numberOfInstances = 0;
  // delete _pInstanceCounts //
  if (_pInstanceCounts != NULL)
  {
    for (int w = 0; w < *_pW; w++)
    {
      for (int n = 0; n < 4; n++)
        delete _pInstanceCounts[w][n];
      delete[] _pInstanceCounts[w]; 
    }
    delete[] _pInstanceCounts; 
  }
  _pInstanceCounts = NULL;  
  // _pW : do not delete (see main destructor)
}
/******************************************************************************
  Method:       AddInstance
  Class:        InstanceMap
  Arguments:    Instance * pInst
  Description:  add a new instance to the list
  Date:         MC_2014/02/15
******************************************************************************/
InstanceMap *
InstanceMap::AddInstance(Instance * pInst)
{
  // add instance at end of list
  if ( pInst != NULL )
  {
    _pInstanceList->push_back(pInst);
    // augment number of instances
    _numberOfInstances++;
    // add to counts
    if (_pInstanceCounts != NULL)
      AddInstanceToCounts(pInst); //
  }
  return this;
}
/******************************************************************************
  Description:  AddInstances(InstanceList *pList)
  Author_Date:  MC_2014/02/15
******************************************************************************/
void
InstanceMap::AddInstances(InstanceList *pList)
{
  if (pList != NULL)
  {
    for (ii = pList->begin(); ii != pList->end(); ii++)
    {
      // add instance at end of list
      AddInstance(*ii);
    }
  }
  return; 
}

/******************************************************************************
  Method:       ClearMap
  Class:        InstanceMap
  Arguments:    none
  Description:  clear all instances in the list and 
                start with a new empty list
  Date:         MC_2014/02/15
******************************************************************************/
void
  InstanceMap::ClearMap()
{
  // first delete the current list
  if ( _pInstanceList != NULL )
  {    
    // delete all individual instances in the list
    list < Instance * >::iterator lIter = _pInstanceList->begin();
    while (lIter != _pInstanceList->end())
    {
      if ( *lIter != NULL )
        delete (*lIter);
      lIter++;
    }
    delete _pInstanceList;
  }
  
  // create a new empty list
  _pInstanceList = new list < Instance * >;
  _numberOfInstances = 0;
	
  // reset _pInstanceCounts //
  if (_pInstanceCounts != NULL)
  {
    for (int w = 0; w < *_pW; w++)
    {
      for (int n = 0; n < 4; n++)
      {
        if ( _pInstanceCounts[w][n] != NULL)
          delete _pInstanceCounts[w][n];
        _pInstanceCounts[w][n] = new StringVector;
      }
    }
  }
  
  return;
}

/******************************************************************************
  Method:       ReplaceInstance
  Class:        InstanceMap
  Arguments:    Instance * oldInst, Instance * newInst
  
  Description:  replace one instance in the list with another instance
  
  Date:         MC_2014/02/15
******************************************************************************/
InstanceMap *
InstanceMap::ReplaceInstance(Instance * oldInst, Instance * newInst)
{
  list < Instance * >::iterator lIter = find(_pInstanceList->begin(),
											 _pInstanceList->end(),oldInst);

  if (lIter != _pInstanceList->end())
  {
    // remove old instance from counts and delete it
    if (_pInstanceCounts != NULL)
      RemoveInstanceFromCounts(*lIter); //
    if ( (*lIter) != NULL )
      delete(*lIter);

    // set new instance and add it to counts
    (*lIter) = newInst;
    if (_pInstanceCounts != NULL)
      AddInstanceToCounts(newInst); //
  }
  else
  {
    // at the end of the list, so add element
    AddInstance(newInst); //
  }
  return this;
}

/******************************************************************************
  Method:       RemoveInstanceFromSequence
  Class:        InstanceMap
  Arguments:    SequenceObject * pSeq
  
  Description:  remove instances from sequence pSeq from the list
  
  Date:         MC_2014/02/15  
******************************************************************************/
InstanceMap *
InstanceMap::RemoveInstanceFromSequence(SequenceObject * pSeq)
{
  //cerr << "Debug-InstanceMap::RemoveInstanceFromSequence -begin" << endl;
  // iterate over all instances
  list < Instance * >::iterator iter = _pInstanceList->begin();
  while (iter != _pInstanceList->end())
  {
    if ( (*iter)->ParentSequence() == pSeq ) // pointers
    {
      if (_pInstanceCounts != NULL)
        RemoveInstanceFromCounts(*iter); //
      // delete instance
      delete (*iter);
      // remove from list
      _pInstanceList->erase(iter);
      _numberOfInstances -= 1; //
      iter = _pInstanceList->begin();// debug
    }
    else {iter++;}
  }
  //cerr << "Debug-InstanceMap::RemoveInstanceFromSequence -end" << endl;
  return this;
}

/******************************************************************************
  Method:       RemoveInstance
  Class:        InstanceMap
  Arguments:    Instance * pInst
  
  Description:  remove a specific instance from the list
  
  Date:         MC_2014/02/15
******************************************************************************/
InstanceMap *
InstanceMap::RemoveInstance(Instance * pInst)
{
  list < Instance * >::iterator iiter = _pInstanceList->begin();
  while (iiter != _pInstanceList->end())
  {
    if ((*iiter) == pInst)
    {
      if (_pInstanceCounts != NULL)
        RemoveInstanceFromCounts(*iiter); //
      delete(*iiter);
      _pInstanceList->erase(iiter);
      _numberOfInstances -= 1; //
      break;  // element is erased leave the while loop
    }
    else
    {
      // next element
      iiter++;
    }
  }
  return this;
}
/******************************************************************************
  Description:  RemoveInstanceFromSequence(string * id)
                remove all instances that have given id as _pSeqName
  Author_Date:  MC_2014/02/26
******************************************************************************/
InstanceMap *
InstanceMap::RemoveInstanceFromSequence(string * id)
{
  // iterate over all instances
  list < Instance * >::iterator InstIter = _pInstanceList->begin();
  while (InstIter != _pInstanceList->end())
  {
    if ( (*((*InstIter)->PrintSeqName())) == *id ) 
    {
     if (_pInstanceCounts != NULL)
        RemoveInstanceFromCounts(*InstIter); //
      // delete instance
      delete (*InstIter);
      // remove from list
      _pInstanceList->erase(InstIter);
      _numberOfInstances -= 1; //
      InstIter = _pInstanceList->begin();// start again to look for more
    }
    else
      InstIter++;
  }
  return this;
}
/******************************************************************************
  Description:  GetIntegerCounts() : return counts matrix of all instances
  Author_Date:  MC_2014/02/15
******************************************************************************/
int**
InstanceMap::GetIntegerCounts()
{
  //cerr << "debug--InstanceMap::GetIntegerCounts() - begin" << endl;
  if (_numberOfInstances == 0) 
  { cerr << "--Warning: InstanceMap::GetIntegerCounts(): no instances with counts." << endl;
    return NULL;
  }
  // this assumes motif length is not changed during NOMS
  // for PHMS the _pW has been set (also assumed fixed) at construction
  int W = 0;
  ii = _pInstanceList->begin();
  if (_pW == NULL) { W =(*ii)->Length();} else { W = *_pW;}
  int ** pCounts = new int*[W]; 
  for (int w = 0; w < W; w++)
  {
    pCounts[w] = new int[4]; 
    for (int n = 0; n < 4; n++)
    pCounts[w][n] = 0; 
  }
  // sum stringvector for each position/each nucleotide
  if (_pInstanceCounts != NULL)
  {
    for (int w = 0; w < W; w++)
    {
      for (int n = 0; n < 4; n++)
       pCounts[w][n] = (int)_pInstanceCounts[w][n]->size();
    }
  }
  else
  { 
    Sequence * pSegment(NULL);
    for (; ii != _pInstanceList->end(); ii++)
    { 
      pSegment = (*ii)->Site(); 
      if (pSegment != NULL)
      { for (int w = 0; w < W; w++)
        { if ((*pSegment)[w] != -1)
          { pCounts[w][(*pSegment)[w]] += 1;}
          else 
          { for (int n = 0; n < 4; n++)
             pCounts[w][n] += 0.25;
          }
        }
      }
      pSegment = NULL;
    } 
  }
  //cerr << "debug--InstanceMap::GetIntegerCounts() - end" << endl;
  return pCounts;
}
/******************************************************************************
  Description:  Add instance to counts
  Author_Date:  MC_2014/02/15
******************************************************************************/
void 
InstanceMap::AddInstanceToCounts(Instance * pInst)
{
  if (pInst != NULL)
  {
    Sequence *pSegment = pInst->Site(); 
    if (pSegment != NULL)
    {
      string speciesid = *(pInst->ParentSequence()->GetID());
      for (int w = 0; w < *_pW; w++)
      {
        if ((*pSegment)[w] != -1)
        {
          // get the stringvector (link pointer)
          StringVector * pVector = _pInstanceCounts[w][(*pSegment)[w]];
          // add species to correct instance-nucleotide
          pVector->push_back(speciesid);
          // sort the vector
          sort(pVector->begin(), pVector->end());
          // reset
          pVector = NULL;
        }
      }
    }
  }
  return;
}

/******************************************************************************
  Description:  Remove instance from counts
  Author_Date:  MC_2014/02/26
******************************************************************************/
void 
InstanceMap::RemoveInstanceFromCounts(Instance * pInst)
{
  //cerr << "Debug-InstanceMap::RemoveInstanceFromCounts -begin" << endl;
  StringVector * pVector = NULL;
  int j;
  if (pInst != NULL)
  {
    Sequence * pSegment = pInst->Site(); 
    if (pSegment != NULL)
    {
      string *pSpeciesid = pInst->ParentSequence()->GetID();
      for (int w = 0; w < *_pW; w++)
      {
        if ((*pSegment)[w] != -1)
        {
          pVector = _pInstanceCounts[w][(*pSegment)[w]];
          // find the position of such species-instance (only one)
          j = 0;
          for (; j < (int)pVector->size(); j++)
          { if ((*pVector)[j] == *pSpeciesid) 
            { break;}
          }
          if (j < (int)pVector->size())
          { pVector->erase(pVector->begin()+j);}
          // sort again
          sort(pVector->begin(), pVector->end());
          // reset
          pVector = NULL;
        }
      }
      pSpeciesid = NULL; 
    }
  }
  pVector = NULL; 
  //cerr << "Debug-InstanceMap::RemoveInstanceFromCounts -end" << endl;
  return;
}
/******************************************************************************
  Method:       Erase
  Class:        InstanceMap
  Arguments:    list<Instance *>::iterator iter
  
  Description:
  
  Date:         MC_2014/02/15
******************************************************************************/
InstanceMap *
InstanceMap::Erase(list<Instance *>::iterator iter)
{
  if ((*iter) != NULL)
  {
    if (_pInstanceCounts != NULL)
      RemoveInstanceFromCounts(*iter); //
    delete(*iter);
    _numberOfInstances--; //
  }
  _pInstanceList->erase(iter);
  return this;
}
