#include "phylutilities.h"


namespace PHYLINCLUSIVE {

/******************************************************************************
  Description:  ReadValue(string value) : read and remove 'value(_)'
  Author_Date:  MC_2009/01/11
******************************************************************************/
  double ReadValue(string &value)
  {
    string::size_type pos;
    double v(-1);
    if ( value.find_first_of("0123456789") != 0)
      throw InvalidInput("phylutilities : numeric input expected");
    
    pos = value.find_first_not_of("0.123456789");
    // convert to double
    istringstream istr(value.substr(0,pos));
    istr >> v;
    if (v < 0 || v == DBL_MAX)
      throw InvalidInput("phylutilities : invalid numeric input");
    
    // erase value and "symbols" and leading whites
    value.erase(0,pos);
    while ( value.find_first_of("_ \t") == 0) value.erase(0,1);

    return v; // should never be -1
  }
/******************************************************************************
  Description:  ReadPriorIndex(string line) : read and remove -p or -M
  Author_Date:  MC_2009/01/12
******************************************************************************/
  bool ReadPriorIndex(string &line)
  {
//  cerr << "debug-phylutilities::ReadPriorIndex - begin" << endl;
    bool i = false;
    // read the index 
    string index = line.substr(0,2);

    if ( index.compare("-p") && index.compare("-M"))
      throw InvalidInput( 
                 "phylutilities : prior >species should start with -p or -M");
    // erase leading -p -M
    if (index == "-p") i = true; // else false = -M
    line.erase(0,2); 

    // erase leading whites
    while ( line.find_first_of("_ \t") == 0)
      line.erase(0,1);
//  cerr << "line = " << line << endl;
//cerr << "debug-phylutilities::ReadPriorIndex - end" << endl;
    return i; 
  }
/******************************************************************************
  Description:  ExtractPriorParameters(Max, Input)
                format 0,u,b,e,f -> 1/2/3/4/5 + parameters 
  Author_Date:  MC_2009/01/27
******************************************************************************/ 
ScoreVector * ExtractPriorParameters(const int &max, string input)
{
//cerr << "debug-phyulilities::ExtractPriorParameters - begin" << endl;
  ScoreVector * pVector = NULL; 
  try 
  {
    char c = input[0]; // 0,u,b,e,f
    switch (c)
    {
      case '0' :
      { // read prior and kappa
        double p = ReadValue(input);
        if (p <= 0 || p >= 1) throw InvalidInput(
                                 "phylutilities : invalid value for -p prior"); 
        double k = 0.25; // kappa SEE DEFAULT IN MAIN
        if ( !input.empty()) 
        { k = ReadValue(input); 
          if (k > 1) throw InvalidInput(
                              "phylutilities : invalid value for -p ..._kappa");
        }
        if (!input.empty()) throw InvalidInput(
                                 "phylutilities : invalid format -p < >_< >");
        // store input
        pVector = new ScoreVector; 
        pVector->push_back(1); pVector->push_back(p); pVector->push_back(k);
        break;
      }
      case 'u' : 
      { input.erase(0,1); // erase 'u'
        if ( !input.empty()) throw InvalidInput(
                                 "phylutilities : invalid format -p u");
        // store input
        pVector = new ScoreVector; pVector->push_back(2);
        break;
      }
      case 'b' : 
      { input.erase(0,1); // erase 'b'
        // read prior input
        double p = ReadValue(input); // prior
        if (p <= 0 || p >= 1) throw InvalidInput(
                                 "phylutilities : invalid value -p b<value>");
        if ( !input.empty()) throw InvalidInput(
                                 "phylutilities : invalid format -p b<value>");
        // store input
        pVector = new ScoreVector; 
        pVector->push_back(3); pVector->push_back(p);
        break;
      }
      case 'e' : 
      { input.erase(0,1); // erase 'e'
        // read all Pr()
        pVector = new ScoreVector; pVector->push_back(4);
        ReadProbabilities(pVector, input, max);
        break;
      }
      case 'f' : 
      { input.erase(0,1); // erase 'f'
        // read all Pr()
        pVector = new ScoreVector; pVector->push_back(5);
//  cerr << "input = " << input << ", max = " << max << endl;
        ReadProbabilities(pVector, input, max);
        break;
      }
      default : throw InvalidInput(
                      "phylutilities : -p input should start with 0,u,b,e,f");
    }
  }
  catch(InvalidInput){ if (pVector != NULL) delete pVector; pVector = NULL;
                       throw ErrorInputfiles(
                                  "phylutilities : in prior -p input"); }
/*
	cerr << "#phylutilities::ExtractPriorParameters : type-";
	cerr << (*pVector)[0] << "( " ;
	for (int i = 1; i < (int)pVector->size(); i++)
		cerr << (*pVector)[i] << " ";
	cerr << "); max=" << max << endl;
*/
//cerr << "debug-phyulilities::ExtractPriorParameters - end" << endl;
  return pVector;
}

/******************************************************************************
  Description:  CreateNbrInstDistribution
                create distributions depending on case
                // max may be reached at a lower value depending -p probs
  Author_Date:  MC_2009/01/27
******************************************************************************/ 
vector<Distribution*> * CreateNbrInstDistribution(const int& max, ScoreVector * Prior)
{
  vector<Distribution*> * pVector = new vector<Distribution*>; 
  pVector->push_back(NULL); // reserved for case f
  ScoreVector * pProbs = NULL;
  try 
  {
    switch ((int)(*Prior)[0])// 1,2,3,4,5
    {
      case 1 :
      { // create vector with probability entries
        pProbs = CreateDefaultProbabilities(max, (*Prior)[1], (*Prior)[2]);
        // create and store distributions
        for (int n = 1; n < (int)pProbs->size(); n++)
          pVector->push_back(new Distribution(pProbs, 0, n+1));
        break;
      }
      case 2 : 
      { // create and store the distributions
        pProbs = new ScoreVector; pProbs->push_back(1);
        for (int n = 1; n <= max; n++) 
        { pProbs->push_back(1);
          pVector->push_back(new Distribution(pProbs, 0, n+1));
        }
        delete pProbs; pProbs = NULL;
        break;
      }
      case 3 : 
      { // create and store the distributions
        for (int n = 1; n <= max; n++)
        { pProbs = CreateBinomialProbabilities((*Prior)[1], n);
          if ( (int)pProbs->size() == n + 1 )
          { pVector->push_back(new Distribution(pProbs, 0, n+1));
            delete pProbs; pProbs = NULL; 
          }
          else break;
        }
        break;
      }
      case 4 : 
      { // create and store the distributions
        bool notNull = false;
        for (int n = 1; n < (int)Prior->size()-1; n++)
        { 
          Distribution * dist = new Distribution(Prior, 1, n+1);
          if (dist != NULL) notNull = true; // some may be NULL when 0 subset
          pVector->push_back(dist); 
        }
        if (!notNull) throw InvalidInput(
                               "phylutilities : too much 0 in prior e<values>");
        break;
      }
      case 5 : 
      { 
        // create and store the distribution
        Distribution * dist = new Distribution(Prior, 1, (int)Prior->size()-1);
        if (dist == NULL) throw InvalidInput(
                              "phylutilities : too much 0 in prior f<values>");
        (*pVector)[0] = dist;
        break;
      }
    }
    // cleanup
    if (pProbs != NULL) delete pProbs; pProbs = NULL;
  }
  catch(InvalidInput){ delete pVector; pVector = NULL;
                       if (pProbs != NULL) delete pProbs; pProbs = NULL;
                       throw ErrorInputfiles(
                                  "phylutilities : in prior -p input"); }
  return pVector;
}
/******************************************************************************
  Description:  CreateDefaultProbabilities : see Gert, untill max or <0.001
  Author_Date:  MC_2009/01/14
******************************************************************************/
  ScoreVector * CreateDefaultProbabilities(const int &m, const double &one, 
                                           const double &kappa)
  {
    double value, sum(1);
    ScoreVector * prob = new ScoreVector;
    prob->push_back(1 - one); // Pr(0)
    prob->push_back(one); // Pr(1)
    // and create other probs
    for(int i = 2; i <= m; i++) 
    { value = kappa * (*prob)[i-1];
      sum += value;
      if ( value/sum <= 0.001) break;
      prob->push_back(value);
    }
    return prob;
  }
/******************************************************************************
  Description:  CreateBinomialProbabilities, cut low values at end
  Author_Date:  MC_2009/01/14
******************************************************************************/
  ScoreVector * CreateBinomialProbabilities(const double &one, const int &m)
  {
    ScoreVector * prob = new ScoreVector;
    // create probabilities
    for(int i = 0; i <= m; i++) 
      prob->push_back(pow(one,i)*pow((1-one),(m-i))*fac(m)/fac(m-i)/fac(i));
    // cut low values at the end
    for(int i = m; i >= 0; i--) 
    { if ( (*prob)[i] <= 0.001) prob->pop_back();
      else break;
    }
    return prob;
  }
/******************************************************************************
  Description:  ReadProbabilities : read maximal m values, cut low at end 
  Author_Date:  MC_2009/01/14
******************************************************************************/
  void ReadProbabilities(ScoreVector * prob, string &line, const int &m)
  {
    // read all Pr()
    double p;
    while (! line.empty() && (int)prob->size() < m+2)
    { p = ReadValue(line); 
      if ( p > 1) throw InvalidInput(
                           "phylutilities : invalid value for -p e|f<values>");
      prob->push_back(p);
    }
    // cut low values at the end
    for(int i = (int)prob->size()-1; i >= 0; i--) 
    { if ( (*prob)[i] <= 0.001) prob->pop_back();
      else break;
    }
    return; 
  } 

/******************************************************************************
  Description:  SegmentLogBackgroundScore
                - modified from SegmentLogBackgroundScore in utilities.h/cpp
  Author_Date:  MC_2009/01/12
******************************************************************************/
  void SegmentLogBackgroundScore(ScoreVector *pbx, 
SequenceObject *pSeq, const strand_modes &s, BackgroundModel *pBgM, const int &W)
  {
    //cerr << "debug : phylutilities::SegmentLogBackgroundScore - begin" << endl; 
    // variables
    int nt, content = 0;
    double sum = 0, value = 0;
    // model parameters
    int order = pBgM->GetOrder();
    int length = pSeq->Length();

    // check the parameters
    if (order >= length || length < W || W < order)
    {
      // so far we do not do anything with the logPbx
      return;
    }
    // relevant startpositions of a segment 
    int Lw = length - W + 1;
    // iterator
    vector <double>::iterator fIter = pbx->begin();
  
    // first segments of the sequence (start < order)
    for (int i = 0; i < order; i++)
    {
      // nt = pSeq->GetNucleotideAt(s, i); //
      sum = 0;
      // use SNF for first positions
      for (int j = i; j < order; j++)
      {
        nt = pSeq->GetNucleotideAt(s, j);
        if (nt != -1)
        {
          value = pBgM->GetSnfValueAt(nt);
          if (value != -1)
            sum += log(value);
          else
            sum += log(0.25);
        }
        else
          sum += log(0.25);
      }
      // use mtrans for further positions > order
      for (int j = order; j < W+i; j++)
      {
        nt = pSeq->GetNucleotideAt(s, j);
        if (nt != -1)
        {
          content = Site2Index(pSeq, s, j - order, order);
          if (content != -1)
          {
            value = pBgM->GetTransitionMatrixValueAt(content, nt);
            if (value != -1)
              sum += log(value);
            else
            {
              value = pBgM->GetSnfValueAt(nt);
              if (value != -1)
                  sum += log(value);
              else
                sum += log(0.25);
            }
          }
          else
              sum += log(0.25);
        }
        else
          sum += log(0.25);
      }
      // update value in pbx
      *(fIter + i) = sum;
    }
  
    // further segments in the sequence ( start > order)
    for (int i = order; i < Lw; i++) // Lw ipv length
    {
      // nt = pSeq->GetNucleotideAt(s, i); // 
      //if (i < Lw)
      //{
      sum = 0;
      for (int j = 0; j < W; j++)
      {
        nt = pSeq->GetNucleotideAt(s, i + j);
        if (nt != -1)
        {
          content = Site2Index(pSeq, s, i + j - order, order);
          if (content != -1)
          {
            value = pBgM->GetTransitionMatrixValueAt(content, nt);
            if (value != -1)
              sum += log(value);
            else
            {
              value = pBgM->GetSnfValueAt(nt);
              sum += log(value);
            }
          }
          else
            sum += log(0.25);
        }
        else
         sum += log(0.25);
      }
      *(fIter + i) = sum;
    }
      /*else
      {
        // cerr << endl << "--Warning-- INCLUSIVE::SegmentLogBackgroundScore() outside computation range." << endl;
        *(fIter + i) = W * log(0.25);
        // this to
      } */
    //cerr << "debug : phylutilities::SegmentLogBackgroundScore - end" << endl;
    return;
  }

/******************************************************************************
                int Site2Index : No changes
  Date:         2003/06/26
  Author:       Gert Thijs <gert.thijs@esat.kuleuven.ac.be>
******************************************************************************/
  int Site2Index(SequenceObject * pSeq, const strand_modes &s, const int &start, const int &order) 
{
    int index = 0;
    int nt = 0;

    if (order < 0) 
    {
      cerr << "Error phylutilities : Site2index   order set to value smaller than 0, returning -1." << endl;
      return -1;
    }

    for (int j = 0; j < order; j++)
    {
      nt = pSeq->GetNucleotideAt(s, start + j);
      if (nt == -1)
      {
        // this is a non ACGT symbol
        index = -1;
        break;
      }
      index += (((int) pow(4.0, order - j - 1)) * nt);
    }
    return index;
  }

  
/******************************************************************************
  Description:  fac(n) = 1 * 2 * 3 * ... * n
  Author_Date:  MC_2007/12/23
******************************************************************************/  int fac(int n)
  {
    int f = 1; int j = n;
    if( j > 1) f = j * fac(j-1);
    return f;
  }

/******************************************************************************
  Description:  SumVector - type vector<double>, start, length
  Author_Date:  MC_2009/01/19
******************************************************************************/
  double SumVector(vector<double> * pVector, int start, int length) 
  {
    double sum = 0;
    for (int i = start; i < start + length; i++)
      sum += (*pVector)[i];
    return sum;
  }

/******************************************************************************
  Description:  SumVector - type vector<double>
  Author_Date:  MC_2009/04/22
******************************************************************************/
  double SumVector(vector<double> * pVector) 
  {
    double sum = 0;
    for (int i = 0; i < (int)pVector->size(); i++)
      sum += (*pVector)[i];
    return sum;
  }

/******************************************************************************
  Description:  order in descending score in a new list
  Author_Date:  MC_2009/04/15
******************************************************************************/
  InstanceList * OrderInstances(InstanceList * list)
  {
    InstanceIterator inst1, inst2, iterEnd;
    // create a new list
    InstanceList *pNew = new InstanceList;
    // link the instances 
    for (inst1 = list->begin(); inst1 != list->end(); inst1++)
      pNew->push_back(*inst1);

    // sort instances as long as not in descending order score
    iterEnd = pNew->end(); iterEnd--;
    bool bSwapped = true;
    while (bSwapped)
    { bSwapped = false;
      for (inst1 = pNew->begin(); inst1 != iterEnd; inst1++)
      { inst2 = inst1; inst2++;
        if ((*inst1)->Score() < (*inst2)->Score()) 
        { swap(*inst1, *inst2); bSwapped = true;}
      }
    }
    return pNew;
  }

/******************************************************************************
  Description:  ComputeLogBetaScore : Beta(ACGT) over motif positions
    // use logaritmic inside this function for Beta
  Author_Date:  MC_2009/04/22
******************************************************************************/
  double ComputeLogBetaScore(double** hw, int *W)
  {
    //cerr << "debug - phylutilities::ComputeLogBetaScore - begin" << endl;
    double logscore(0);
    // iterate through positions
    for (int j = 0; j < *W; j++)
	{
      // logaritmic Beta(A,C,G,T) over motif width positions
      logscore += LogBeta(hw[j]);
    }
    return logscore; // logaritmic
  }
/******************************************************************************
  Description:  LogGamma function for x = double
  Author_Date:  MC_2009/03/27
******************************************************************************/
  double LogGamma(const double &x)
  {
    static const double coefficients[6] = 
    {
      76.18009172947146,
      -86.50532032941677,
      24.01409824083091,
      -1.231739572450155,
      0.1208650973866179e-2,
      -0.5395239384953e-5
    };
    double 
      t1 = x,
      t2 = x,
      t3 = 1.000000000190015,
      value = 0;
    
    t2 += 5.5;
    t2 -= (x+0.5)* log(t2);
    for (int j=0; j<6; j++)
      t3 += coefficients[j]/(++t1);
    
    value = -t2 + log(2.5066282746310005 * t3 / x);    
    return value;
  }
/******************************************************************************
  Description:  Logaritmic Beta function for hwj(double*) (A,C,G,T) 
  Author_Date:  MC_2009/03/30
******************************************************************************/
  double 
  LogBeta(double* pWeightj)
  {
    // size of pWeightj == 4 (A,C,G,T) 
    double value(0), sum(0);
    for(int n = 0; n < 4; n++)
    {
      value += LogGamma(pWeightj[n]);
      sum += pWeightj[n];
    }
    value -= LogGamma(sum);

    return value; // logaritmic Beta
  }
  
/******************************************************************************
  Description:  compute the inverse of a 4x4 matrix (code copied from internet)
  input of the matrix is as double[16] in column order
  Author_Date:  MC_2012/11/28
******************************************************************************/
  bool gluInvertMatrix(const double m[16], double invOut[16])
  {
	//cerr << "phylutilities::gluInvertMatrix-begin" << endl;
    double inv[16], det;
    int i;

    inv[0] = m[5]  * m[10] * m[15] - 
             m[5]  * m[11] * m[14] - 
             m[9]  * m[6]  * m[15] + 
             m[9]  * m[7]  * m[14] +
             m[13] * m[6]  * m[11] - 
             m[13] * m[7]  * m[10];

    inv[4] = -m[4]  * m[10] * m[15] + 
              m[4]  * m[11] * m[14] + 
              m[8]  * m[6]  * m[15] - 
              m[8]  * m[7]  * m[14] - 
              m[12] * m[6]  * m[11] + 
              m[12] * m[7]  * m[10];

    inv[8] = m[4]  * m[9] * m[15] - 
             m[4]  * m[11] * m[13] - 
             m[8]  * m[5] * m[15] + 
             m[8]  * m[7] * m[13] + 
             m[12] * m[5] * m[11] - 
             m[12] * m[7] * m[9];

    inv[12] = -m[4]  * m[9] * m[14] + 
               m[4]  * m[10] * m[13] +
               m[8]  * m[5] * m[14] - 
               m[8]  * m[6] * m[13] - 
               m[12] * m[5] * m[10] + 
               m[12] * m[6] * m[9];

    inv[1] = -m[1]  * m[10] * m[15] + 
              m[1]  * m[11] * m[14] + 
              m[9]  * m[2] * m[15] - 
              m[9]  * m[3] * m[14] - 
              m[13] * m[2] * m[11] + 
              m[13] * m[3] * m[10];

    inv[5] = m[0]  * m[10] * m[15] - 
             m[0]  * m[11] * m[14] - 
             m[8]  * m[2] * m[15] + 
             m[8]  * m[3] * m[14] + 
             m[12] * m[2] * m[11] - 
             m[12] * m[3] * m[10];

    inv[9] = -m[0]  * m[9] * m[15] + 
              m[0]  * m[11] * m[13] + 
              m[8]  * m[1] * m[15] - 
              m[8]  * m[3] * m[13] - 
              m[12] * m[1] * m[11] + 
              m[12] * m[3] * m[9];

    inv[13] = m[0]  * m[9] * m[14] - 
              m[0]  * m[10] * m[13] - 
              m[8]  * m[1] * m[14] + 
              m[8]  * m[2] * m[13] + 
              m[12] * m[1] * m[10] - 
              m[12] * m[2] * m[9];

    inv[2] = m[1]  * m[6] * m[15] - 
             m[1]  * m[7] * m[14] - 
             m[5]  * m[2] * m[15] + 
             m[5]  * m[3] * m[14] + 
             m[13] * m[2] * m[7] - 
             m[13] * m[3] * m[6];

    inv[6] = -m[0]  * m[6] * m[15] + 
              m[0]  * m[7] * m[14] + 
              m[4]  * m[2] * m[15] - 
              m[4]  * m[3] * m[14] - 
              m[12] * m[2] * m[7] + 
              m[12] * m[3] * m[6];

    inv[10] = m[0]  * m[5] * m[15] - 
              m[0]  * m[7] * m[13] - 
              m[4]  * m[1] * m[15] + 
              m[4]  * m[3] * m[13] + 
              m[12] * m[1] * m[7] - 
              m[12] * m[3] * m[5];

    inv[14] = -m[0]  * m[5] * m[14] + 
               m[0]  * m[6] * m[13] + 
               m[4]  * m[1] * m[14] - 
               m[4]  * m[2] * m[13] - 
               m[12] * m[1] * m[6] + 
               m[12] * m[2] * m[5];

    inv[3] = -m[1] * m[6] * m[11] + 
              m[1] * m[7] * m[10] + 
              m[5] * m[2] * m[11] - 
              m[5] * m[3] * m[10] - 
              m[9] * m[2] * m[7] + 
              m[9] * m[3] * m[6];

    inv[7] = m[0] * m[6] * m[11] - 
             m[0] * m[7] * m[10] - 
             m[4] * m[2] * m[11] + 
             m[4] * m[3] * m[10] + 
             m[8] * m[2] * m[7] - 
             m[8] * m[3] * m[6];

    inv[11] = -m[0] * m[5] * m[11] + 
               m[0] * m[7] * m[9] + 
               m[4] * m[1] * m[11] - 
               m[4] * m[3] * m[9] - 
               m[8] * m[1] * m[7] + 
               m[8] * m[3] * m[5];

    inv[15] = m[0] * m[5] * m[10] - 
              m[0] * m[6] * m[9] - 
              m[4] * m[1] * m[10] + 
              m[4] * m[2] * m[9] + 
              m[8] * m[1] * m[6] - 
              m[8] * m[2] * m[5];

    det = m[0] * inv[0] + m[1] * inv[4] + m[2] * inv[8] + m[3] * inv[12];

    if (det == 0)
        return false;

    det = 1.0 / det;

    for (i = 0; i < 16; i++)
        invOut[i] = inv[i] * det;
	//cerr << "phylutilities::gluInvertMatrix-end" << endl;
    return true;
  }
} // end of the namespace
